import 'dart:async';
import 'dart:io' show Platform;

import 'package:flutter/services.dart';

///NFC 状态
enum NFCStatus {
  none,
  reading,
  read,
  stopped,
  error,
}

///NFC 数据
class NfcData {
  final String id;
  final String content;
  final String error;
  final String statusMapper;

  NFCStatus status;

  NfcData({
    this.id,
    this.content,
    this.error,
    this.statusMapper,
  });

  factory NfcData.fromMap(Map data) {
    NfcData result = NfcData(
      id: data['nfcId'],
      content: data['nfcContent'],
      error: data['nfcError'],
      statusMapper: data['nfcStatus'],
    );
    switch (result.statusMapper) {
      case 'none':
        result.status = NFCStatus.none;
        break;
      case 'reading':
        result.status = NFCStatus.reading;
        break;
      case 'stopped':
        result.status = NFCStatus.stopped;
        break;
      case 'error':
        result.status = NFCStatus.error;
        break;
      default:
        result.status = NFCStatus.none;
    }
    return result;
  }
}

///NFC 类
class FlutterNfc {
  static const MethodChannel _channel = const MethodChannel('flutter_nfc');
  static const stream = const EventChannel('com.example.flutter_nfc');

  static Future<NfcData> stop() async {
    final Map data = await _channel.invokeMethod('NfcStop');
    final NfcData result = NfcData.fromMap(data);

    return result;
  }

  static Future<NfcData> read() async {
    final Map data = await _channel.invokeMethod('NfcRead');
    final NfcData result = NfcData.fromMap(data);

    return result;
  }

  static Stream<NfcData> onTagDiscovered() {
    if (Platform.isIOS) {
      _channel.invokeMethod('NfcRead');
    }
    return stream.receiveBroadcastStream().map((rawNfcData) {
      return NfcData.fromMap(rawNfcData);
    });
  }

  static Future<NfcData> write(String path, String label) async {
    final Map data = await _channel.invokeMethod(
        'NfcWrite', <String, dynamic>{'label': label, 'path': path});

    final NfcData result = NfcData.fromMap(data);

    return result;
  }
}
